==> libinklevel-0.6.3.tar.gz: libinklevel/linux.c <==
/* linux.c
 *
 * (c) 2003, 2004 Markus Heinz
 *
 * This software is licensed under the terms of the GPL.
 * For details see file COPYING.
 */

#include "inklevel.h"

#include <ieee1284.h>

#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <string.h>
#include <sys/ioctl.h>

#define IOCNR_GET_DEVICE_ID 1
#define LPIOC_GET_DEVICE_ID _IOC(_IOC_READ, 'P', IOCNR_GET_DEVICE_ID, BUFLEN)

int get_device_id(int, int, unsigned char *);

/* This function retrieves the device id of the specified port */

int get_device_id(int port, int portnumber, unsigned char *device_id) {
  struct parport_list parports;
  unsigned char tmp[BUFLEN];
  unsigned char device_file[256];
  int size;
  int fd;
  unsigned char *c;
  int realsize;

  if (port == PARPORT ) {
    /* check if we have appropiate permissions */

    sprintf(device_file, "/dev/parport%d", portnumber);
    if ((fd = open(device_file, O_RDWR)) < 0) {
      return DEV_PARPORT_INACCESSIBLE;
    }
    close(fd);

    sprintf(device_file, "/dev/lp%d", portnumber);
    if ((fd = open(device_file, O_RDWR)) < 0) {
      return DEV_LP_INACCESSIBLE;
    }
    close(fd);

    if (ieee1284_find_ports(&parports, 0) == E1284_OK) {
      if (portnumber < parports.portc) {
	size = ieee1284_get_deviceid(parports.portv[portnumber], -1, 
                                     F1284_FRESH, tmp, BUFLEN);
	if (size >= 2) {
	  strncpy(device_id, tmp + 2, size - 2);
	  return OK;
	}
      }
    }
    return COULD_NOT_GET_DEVICE_ID;
  } else if (port == USB) {
    sprintf(device_file, "/dev/usb/lp%d", portnumber);
    fd = open(device_file, O_RDONLY);
    if (fd == ERROR) {
      return DEV_USB_LP_INACCESSIBLE;
    }
    if (ioctl(fd, LPIOC_GET_DEVICE_ID, tmp) < 0) {
      close(fd);
      return COULD_NOT_GET_DEVICE_ID;
    }
    close(fd);
    size = (tmp[0] << 8) | (tmp[1]);

    /* Some printers report the size of the device id incorrectly. */

    realsize = 2;
    c = &tmp[2];
    while (*c++ != '\0') {
      realsize++;
    }

#ifdef DEBUG
    printf("Printer reported size %d, real size is %d\n", size, realsize);
#endif

    size = (realsize < size) ? realsize : size;
    size = (size < BUFLEN - 1) ? size : BUFLEN - 1;
    tmp[size] = '\0';
    if (size >= 2) {
      strncpy(device_id, tmp + 2, size - 2);
      return OK;
    } else {
      return COULD_NOT_GET_DEVICE_ID;
    }
  } else {
    return UNKNOWN_PORT_SPECIFIED;
  }
}

==> libinklevel-0.6.5rc1.tar.gz: libinklevel/linux.c <==
/* linux.c
 *
 * (c) 2003, 2004, 2005 Markus Heinz
 *
 * This software is licensed under the terms of the GPL.
 * For details see file COPYING.
 */

#include "inklevel.h"

#include <ieee1284.h>

#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <string.h>
#include <sys/ioctl.h>

#define IOCNR_GET_DEVICE_ID 1
#define LPIOC_GET_DEVICE_ID _IOC(_IOC_READ, 'P', IOCNR_GET_DEVICE_ID, BUFLEN)

int get_device_id(int port, int portnumber, unsigned char *device_id);

/* This function retrieves the device id of the specified port */

int get_device_id(int port, int portnumber, unsigned char *device_id) {
  struct parport_list parports;
  unsigned char tmp[BUFLEN];
  unsigned char device_file[256];
  int size;
  int fd;
  unsigned char *c;
  int realsize;

  if (port == PARPORT ) {
    /* check if we have appropiate permissions */

    sprintf(device_file, "/dev/parport%d", portnumber);
    if ((fd = open(device_file, O_RDWR)) < 0) {
      return DEV_PARPORT_INACCESSIBLE;
    }
    close(fd);

    sprintf(device_file, "/dev/lp%d", portnumber);
    if ((fd = open(device_file, O_RDWR)) < 0) {
      return DEV_LP_INACCESSIBLE;
    }
    close(fd);

    if (ieee1284_find_ports(&parports, 0) == E1284_OK) {
      if (portnumber < parports.portc) {
	size = ieee1284_get_deviceid(parports.portv[portnumber], -1, 
                                     F1284_FRESH, tmp, BUFLEN);
	if (size >= 2) {
	  strncpy(device_id, tmp + 2, size - 2);
	  return OK;
	}
      }
    }
    return COULD_NOT_GET_DEVICE_ID;
  } else if (port == USB) {
    sprintf(device_file, "/dev/usb/lp%d", portnumber);
    fd = open(device_file, O_RDONLY);
    if (fd == ERROR) {
      return DEV_USB_LP_INACCESSIBLE;
    }
    if (ioctl(fd, LPIOC_GET_DEVICE_ID, tmp) < 0) {
      close(fd);
      return COULD_NOT_GET_DEVICE_ID;
    }
    close(fd);
    size = (tmp[0] << 8) | (tmp[1]);

    /* Some printers report the size of the device id incorrectly. */

    realsize = 2;
    c = &tmp[2];
    while (*c++ != '\0') {
      realsize++;
    }

#ifdef DEBUG
    printf("Printer reported size %d, real size is %d\n", size, realsize);
#endif

    size = (realsize < size) ? realsize : size;
    size = (size < BUFLEN - 1) ? size : BUFLEN - 1;
    tmp[size] = '\0';
    if (size >= 2) {
      strncpy(device_id, tmp + 2, size - 2);
      return OK;
    } else {
      return COULD_NOT_GET_DEVICE_ID;
    }
  } else {
    return UNKNOWN_PORT_SPECIFIED;
  }
}
